#!/usr/bin/env python
#----------------------------------------------------------------------------
# Name:         OritatamiShapeMaker.py
# Purpose:      A basic program for drawing shapes, scaling and rotating them,
#               and concatenating sets of bead sequences to create an Oritatami
#               system which self-assembles the shape (at constant scale factor
#               and with a constant set of bead types, but with a sequence which
#               grows linearly with the size of the shape).
#
# Author:       Matt Patitz
#
# Created:      
# RCS-ID:       
# Copyright:    
# License:      
#----------------------------------------------------------------------------

"""
This program can load the definition of a shape system from a file and
generate an Oritatami system which self-assembles a scaled version of it.

Requirements:
    This application requires Python 2.x

Dependencies:
    wxPython    (download installer for your platform here: https://wxpython.org/Phoenix/snapshot-builds/)
    pil         (via pillow, install using 'pip install pillow', pip can be found in your Python27\Scripts dir)
"""


"""
MJP TODO list:
    - verify that each node only has two edges after creating Hamiltonian cycle
    - improve fast forward functionality to be multi-threaded
    - BUG:  Toggling the grid and zooming can cause the grid to be shifted partially off of the visible window area
"""


import wx
import wx.richtext as rt
import os
import random
import math
import re
import ast # TODO: This was needed for debugging remove when done using 
import sys
import thread
from collections import deque
import itertools
import wx.aui
import time
import threading
import wx.lib.scrolledpanel as scrolled
#from wx.lib.agw.shapedbutton import SButton, SBitmapButton
import wx.lib.agw.cubecolourdialog as CCD
import wx.html
from LookupTable import *
import ShapeMakerNodes
import ShapeMakerUtils
import ShapeMakerGrid
import ShapeMakerCanvas
from SimulatorFramework import SimApp


# IDs
ORIT_ID_FR             = 100
ORIT_ID_REV            = 101
ORIT_ID_STOP           = 102
ORIT_ID_FOR            = 103
ORIT_ID_FF             = 104
ORIT_ID_NEW            = 105
ORIT_ID_LOAD           = 106
ORIT_ID_SAVE           = 107
ORIT_ID_ZOOMIN         = 108
ORIT_ID_ZOOMOUT        = 109
ORIT_ID_MODE_PAN_ZOOM  = 110
ORIT_ID_MODE_DRAW_BEAD = 111
ORIT_ID_MODE_DRAW_LINK = 112
ORIT_ID_MODE_DRAW_BOND = 113
ORIT_ID_RESET          = 114
ORIT_ID_RESET_BEADSEQ  = 115
ORIT_ID_RELOAD_BEADSEQ = 116
ORIT_ID_CANCEL_BEADSEQ = 117
ORIT_ID_OK_NOCHECK     = 118
ORIT_ID_OK_CHECK       = 119
ORIT_ID_CANCEL         = 120
ORIT_ID_COMPILE        = 121
ORIT_ID_GENOS          = 122

ORIT_MODE_PAN_ZOOM     = 0
ORIT_MODE_DRAW_BEAD    = 1
ORIT_MODE_DRAW_LINK    = 2
ORIT_MODE_DRAW_BOND    = 3


# Result statuses for stepping forward in simulation
ORI_STAT_OK         = 0
ORI_STAT_NO_SPACE   = 1
ORI_STAT_NO_BINDING = 2
ORI_STAT_NONDET     = 3
ORI_STAT_MAX_FLOPPY = 4
ORI_STAT_ERROR      = 5




colours = [
    "BLACK",
    "BLUE",
    "BLUE VIOLET",
    "BROWN",
    "CYAN",
    "DARK GREY",
    "DARK GREEN",
    "GOLD",
    "GREY",
    "GREEN",
    "MAGENTA",
    "NAVY",
    "PINK",
    "RED",
    "SKY BLUE",
    "VIOLET",
    "YELLOW",
    ]


DIRSTRS = ("N", "E", "S", "W")

#assertMode = wx.PYAPP_ASSERT_DIALOG
##assertMode = wx.PYAPP_ASSERT_EXCEPTION

#globalApp = None


class OritatamiSimulatorApp(SimApp):
    def __init__(self):
        SimApp.__init__(self)
        self.showMessages = True
        self.messageQueue = []
        self.showDisplayUpdates = True
        self.useAdvancedColorPicker = False

    def OnInit(self):
        self.messageWin             = None
        self.systemDefinitionFile   = None
        self.systemDefinitionFileFullPath = None
        self.seedDefinitionFile     = None
        self.assemblyDefinitionFile = None

        return SimApp.OnInit(self, title="Oritatami Shape Maker")

    def CreateCanvas(self):
        self.canvas = ShapeMakerCanvas.ShapeMakerCanvas(self.frame, wx.ID_ANY, self.frame, wx.DefaultSize, self)
        self.SetMouseMode(ORIT_ID_MODE_DRAW_BEAD)

    def CreateMenu(self):
        menuBar = wx.MenuBar()
        menu = wx.Menu()
        itemNew = menu.Append(-1, "New shape definition", "New shape definition")
        self.Bind(wx.EVT_MENU, self.OnNew, itemNew)
        menu.AppendSeparator()
        #itemLSys = menu.Append(-1, "Load system definition", "Load system definition")
        #self.Bind(wx.EVT_MENU, self.OnLoadSystem, itemLSys)
        item1 = menu.Append(-1, "&Load shape definition\tCtrl-L", "Load shape definition")
        self.Bind(wx.EVT_MENU, self.OnLoadAssembly, item1)
        itemR = menu.Append(-1, "Reload shape definition", "Reload shape definition")
        self.Bind(wx.EVT_MENU, self.OnReload, itemR)
        itemReset = menu.Append(-1, "Reset To shape definition", "Reset to shape definition")
        self.Bind(wx.EVT_MENU, self.OnReset, itemReset)
        menu.AppendSeparator()
        #itemR = menu.Append(-1, "&Reset to seed\tCtrl-R", "Reset system to the seed")
        #self.Bind(wx.EVT_MENU, self.OnReset, itemR)
        menu.AppendSeparator()
        itemSBT = menu.Append(-1, "Save shape definition", "Save shape definition")
        self.Bind(wx.EVT_MENU, self.OnSaveShapeDef, itemSBT)
        itemSBTA = menu.Append(-1, "Save shape definition as..", "Save shape definition as..")
        self.Bind(wx.EVT_MENU, self.OnSaveShapeDefAs, itemSBTA)
        menu.AppendSeparator()
        #itemSetSeed = menu.Append(-1, "Set current assembly as seed", "Set current assembly as seed assembly")
        #self.Bind(wx.EVT_MENU, self.OnSetSeed, itemSetSeed)
        #itemSS = menu.Append(-1, "Save seed", "Save seed assembly")
        #self.Bind(wx.EVT_MENU, self.OnSaveSeed, itemSS)
        #itemSSA = menu.Append(-1, "Save seed as..", "Save seed assembly as..")
        #self.Bind(wx.EVT_MENU, self.OnSaveSeedAs, itemSSA)
        #menu.AppendSeparator()
        #itemSA = menu.Append(-1, "Save current assembly", "Save current assembly")
        #self.Bind(wx.EVT_MENU, self.OnSaveAssembly, itemSA)
        #itemSAA = menu.Append(-1, "Save current assembly as..", "Save current assembly as..")
        #self.Bind(wx.EVT_MENU, self.OnSaveAssemblyAs, itemSAA)
        #menu.AppendSeparator()
        #item2 = menu.Append(-1, "&Save system definition and seed\tCtrl-S", "Save system definition and seed assembly")
        #self.Bind(wx.EVT_MENU, self.OnSaveAll, item2)
        menu.AppendSeparator()
        item3 = menu.Append(-1, "E&xit\tCtrl-X", "Exit program")
        self.Bind(wx.EVT_MENU, self.OnExitApp, item3)
        menuBar.Append(menu, "&File")

        menuCtrl = wx.Menu()
        #itemBP = menuCtrl.Append(-1, "&Breakpoints\tCtrl-B", "Set breakpoints")
        #self.Bind(wx.EVT_MENU, self.OnBreakpoints, itemBP)
        #menuCtrl.AppendSeparator()
        itemStep = menuCtrl.Append(-1, "&Forward\tCtrl-F", "Forward one step")
        self.Bind(wx.EVT_MENU, self.OnForward, itemStep)
        itemFF = menuCtrl.Append(-1, "&Go\tCtrl-G", "Fast forward")
        self.Bind(wx.EVT_MENU, self.OnGo, itemFF)
        #itemQuit = menuCtrl.Append(-1, "&Quit\tCtrl-Q", "Quit fast forward")
        #self.Bind(wx.EVT_MENU, self.OnQuit, itemQuit)
        menuBar.Append(menuCtrl, "&Control")

        self.menuView = wx.Menu()
        self.itemShowMess = self.menuView.Append(-1, "Show message window", "Toggle display of message window", kind=wx.ITEM_CHECK)
        self.Bind(wx.EVT_MENU, self.OnShowMessageWin, self.itemShowMess)
        self.menuView.Check(self.itemShowMess.GetId(), True)
        self.menuView.AppendSeparator()
        itemSTC = self.menuView.Append(-1, "&Center\tCtrl+C", "Scroll to center")
        self.Bind(wx.EVT_MENU, self.OnCenter, itemSTC)
        self.menuView.AppendSeparator()
        itemZIn = self.menuView.Append(-1, "Zoom in\tCtrl-+", "Zoom in")
        self.Bind(wx.EVT_MENU, self.OnZoomIn, itemZIn)
        itemZOut = self.menuView.Append(-1, "Zoom out\tCtrl--", "Zoom out")
        self.Bind(wx.EVT_MENU, self.OnZoomOut, itemZOut)
        self.menuView.AppendSeparator()
        itemTG = self.menuView.Append(-1, "&Toggle grid lines\tCtrl-T", "Toggle display of grid lines")
        self.Bind(wx.EVT_MENU, self.OnToggleGrid, itemTG)
        self.itemTM = self.menuView.Append(-1, "Update &messages\tCtrl-M", "Toggle display of messages", kind=wx.ITEM_CHECK)
        self.Bind(wx.EVT_MENU, self.OnMessageToggle, self.itemTM)
        self.menuView.Check(self.itemTM.GetId(), True)
        self.itemTD = self.menuView.Append(-1, "Update &display\tCtrl-D", "Toggle display of assembly updates", kind=wx.ITEM_CHECK)
        self.Bind(wx.EVT_MENU, self.OnDisplayToggle, self.itemTD)
        self.menuView.Check(self.itemTD.GetId(), True)
        self.menuView.AppendSeparator()
        itemRefresh = self.menuView.Append(-1, "Refresh screen\tF2", "Refresh screen")
        self.Bind(wx.EVT_MENU, self.OnRefresh, itemRefresh)
        menuBar.Append(self.menuView, "&View")

        #self.menuSettings = wx.Menu()
        #self.itemAdvColor = self.menuSettings.Append(-1, "Use advanced color picker", "Use advanced color picker dialog", kind=wx.ITEM_CHECK)
        #self.Bind(wx.EVT_MENU, self.OnAdvancedColorChecked, self.itemAdvColor)
        #self.menuSettings.Check(self.itemAdvColor.GetId(), False)
        #menuBar.Append(self.menuSettings, "&Settings")
    
        menu2 = wx.Menu()
        #itemDoc = menu2.Append(-1, "Documentation", "Show documentation")
        #self.Bind(wx.EVT_MENU, self.OnDoc, itemDoc)
        itemH = menu2.Append(-1, "&Help\tCtrl-H", "Show help")
        self.Bind(wx.EVT_MENU, self.OnHelp, itemH)
        menuBar.Append(menu2, "&Help")
        self.frame.SetMenuBar(menuBar)
        
    
    def CreateToolbar(self):
        tb1 = wx.ToolBar(self.frame, -1, wx.DefaultPosition, wx.DefaultSize, wx.TB_DOCKABLE | wx.TB_FLAT)
        tb1.SetToolSeparation(20)
        tb1.SetToolBitmapSize(wx.Size(24,24))
        tb1.AddTool(ORIT_ID_NEW, "New System", wx.Image("images/new.xpm", wx.BITMAP_TYPE_XPM).ConvertToBitmap(), shortHelp="New System")#, longHelp="New oritatami system")
        tb1.AddTool(ORIT_ID_LOAD, "Load System", wx.Image("images/load.xpm", wx.BITMAP_TYPE_XPM).ConvertToBitmap(), shortHelp="Load System")#, longHelp="Load oritatami system")
        tb1.AddTool(ORIT_ID_SAVE, "Save System", wx.Image("images/save.xpm", wx.BITMAP_TYPE_XPM).ConvertToBitmap(), shortHelp="Save System")#, longHelp="Save oritatami system")
        tb1.AddTool(ORIT_ID_RESET, "Reset To Shape", wx.Image("images/reset.xpm", wx.BITMAP_TYPE_XPM).ConvertToBitmap(), shortHelp="Reset To Shape Definition")#, longHelp="Reset to shape definition")
        tb1.Realize()
        tb1.Bind(wx.EVT_TOOL, self.OnTool)

        self.frame._mgr.AddPane(tb1, wx.aui.AuiPaneInfo().
                          Name("File toolbar").Caption("File Toolbar").
                          ToolbarPane().Top())
        #tb1.AddSeparator()
        #tb1.AddSeparator()
        
        tb1 = wx.ToolBar(self.frame, -1, wx.DefaultPosition, wx.DefaultSize, wx.TB_DOCKABLE | wx.TB_FLAT)
        tb1.SetToolSeparation(20)
        tb1.SetToolBitmapSize(wx.Size(24,24))
        btnImg = wx.Image("images/redo.xpm", wx.BITMAP_TYPE_XPM)
        btnImg.Rescale(25, 15)
        tb1.AddTool(ORIT_ID_COMPILE, "Scale and rotate shape", btnImg.ConvertToBitmap(), shortHelp="Scale and rotate shape")#, longHelp="Scale and rotate shape")

        #tb1.AddTool(ORIT_ID_FR, "Fast Rewind", wx.Image("images/fastleft.xpm", wx.BITMAP_TYPE_XPM).ConvertToBitmap(), shortHelp="Fast Rewind")#, longHelp="Fast rewind")
        #tb1.AddTool(ORIT_ID_REV, "Rewind", wx.Image("images/left.xpm", wx.BITMAP_TYPE_XPM).ConvertToBitmap(), shortHelp="Rewind")#, longHelp="Rewind one step")
        #tb1.AddTool(ORIT_ID_STOP, "Pause", wx.Image("images/pause.xpm", wx.BITMAP_TYPE_XPM).ConvertToBitmap(), shortHelp="Pause")#, longHelp="Pause simulation")
        tb1.AddTool(ORIT_ID_FOR, "Forward (add one gadget)", wx.Image("images/right.xpm", wx.BITMAP_TYPE_XPM).ConvertToBitmap(), shortHelp="Forward (add one gadget)")#, longHelp="Forward one step")
        tb1.AddTool(ORIT_ID_FF, "Fast Forward (add all gadgets)", wx.Image("images/fastright.xpm", wx.BITMAP_TYPE_XPM).ConvertToBitmap(), shortHelp="Fast Forward (add all gadgets)")#, longHelp="Fast forward")
        btnImg = wx.Image("images/logo.png", wx.BITMAP_TYPE_PNG)
        btnImg.Rescale(25, 15)
        tb1.AddTool(ORIT_ID_GENOS, "Generate Oritatami system", btnImg.ConvertToBitmap(), shortHelp="Generate Oritatami system")#, longHelp="Generate Oritatami system")

        tb1.AddSeparator()
        tb1.AddTool(ORIT_ID_ZOOMIN, "Zoom In", wx.Image("images/zoomin.xpm", wx.BITMAP_TYPE_XPM).ConvertToBitmap(), shortHelp="Zoom In")#, longHelp="Zoom in")
        tb1.AddTool(ORIT_ID_ZOOMOUT, "Zoom Out", wx.Image("images/zoomout.xpm", wx.BITMAP_TYPE_XPM).ConvertToBitmap(), shortHelp="Zoom Out")#, longHelp="Zoom out")
        tb1.Realize()
        tb1.Bind(wx.EVT_TOOL, self.OnTool)

        self.frame._mgr.AddPane(tb1, wx.aui.AuiPaneInfo().
                          Name("Compilation toolbar").Caption("Compilation Toolbar").
                          ToolbarPane().Top())

        tb1 = wx.ToolBar(self.frame, -1, wx.DefaultPosition, wx.DefaultSize, wx.TB_DOCKABLE | wx.TB_FLAT)
        tb1.SetToolSeparation(10)
        #tb1.SetToolPacking(10)
        #tb1.SetToolBitmapSize(wx.Size(16,16))
        btnImg = wx.Image("images/add-remove-bead.png", wx.BITMAP_TYPE_PNG)
        btnImg.Rescale(25, 15)
        tb1.AddRadioTool(ORIT_ID_MODE_DRAW_BEAD, "Add/Remove", btnImg.ConvertToBitmap(), shortHelp="Add beads (left mouse)/remove beads (right mouse)", longHelp="Add beads (left mouse) and remove beads (right mouse)")

        btnImg = wx.Image("images/pan-zoom.png", wx.BITMAP_TYPE_PNG)
        btnImg.Rescale(45, 25)
        tb1.AddRadioTool(ORIT_ID_MODE_PAN_ZOOM, "Zoom/Pan", btnImg.ConvertToBitmap(), shortHelp="Pan (left mouse)/zoom (right mouse)", longHelp="Pan (left mouse) and zoom (right mouse)")
        btnImg.Rescale(25, 15)
        tb1.Realize()
        tb1.Bind(wx.EVT_TOOL, self.OnTool)

        self.frame._mgr.AddPane(tb1, wx.aui.AuiPaneInfo().
                          Name("Mouse mode toolbar").Caption("Mouse Mode Toolbar").
                          ToolbarPane().Top())

    def OnTool(self, evt):
        if evt.GetId() == ORIT_ID_MODE_PAN_ZOOM:
            self.SetMouseMode(ORIT_ID_MODE_PAN_ZOOM)
        elif evt.GetId() == ORIT_ID_MODE_DRAW_BEAD:
            self.SetMouseMode(ORIT_ID_MODE_DRAW_BEAD)
        elif evt.GetId() == ORIT_ID_NEW:
            self.canvas.DoNew()
        elif evt.GetId() == ORIT_ID_LOAD:
            self.canvas.DoLoad(bAssembly=True, bSystem=True)
        elif evt.GetId() == ORIT_ID_RESET:
            self.canvas.Reset()
        elif evt.GetId() == ORIT_ID_SAVE:
            self.canvas.DoSaveAll()
        elif evt.GetId() == ORIT_ID_COMPILE:
            self.canvas.Compile()
        elif evt.GetId() == ORIT_ID_GENOS:
            self.canvas.GenerateOritatamiSystem()
        elif evt.GetId() == ORIT_ID_FOR:
            self.canvas.DoForwardStep()
        elif evt.GetId() == ORIT_ID_FF:
            self.canvas.DoGo()
        elif evt.GetId() == ORIT_ID_ZOOMIN:
            self.canvas.ZoomThenRecenter(True)
        elif evt.GetId() == ORIT_ID_ZOOMOUT:
            self.canvas.ZoomThenRecenter(False)
        
    def OnShowMessageWin(self, event):
        pane = self.frame._mgr.GetPane(self.ctrlPanel)
        if self.itemShowMess.IsChecked():
            pane.Show()
        else:
            pane.Hide()
        self.frame._mgr.Update()
        
    def OnAdvancedColorChecked(self, event):
        if self.itemShowMess.IsChecked():
            self.useAdvancedColorPicker = True
        else:
            self.useAdvancedColorPicker = False
        
    def SetMouseMode(self, mode):
        if not mode == self.canvas.mouseMode:
            self.canvas.linkStart = None
            self.canvas.linkEnd = None
            if (self.canvas.posSelected is not None) or (self.canvas.negSelected is not None):
                self.canvas.posSelected = None
                self.canvas.negSelected = None
                self.canvas.InitializeDrawing(self.canvas.pdc)
        if mode == ORIT_ID_MODE_PAN_ZOOM:
            self.canvas.mouseMode = ORIT_MODE_PAN_ZOOM
        elif mode == ORIT_ID_MODE_DRAW_BEAD:
            self.canvas.mouseMode = ORIT_MODE_DRAW_BEAD

    #===========================================================================
    # def SetOutputMessage(self, msg):
    #     if not self.showMessages:
    #         self.messageQueue.append(msg)
    #         return
    #     self.messageWin.SetInsertionPointEnd()
    #     self.messageWin.WriteText("\n" + msg)
    #     self.messageWin.SetInsertionPointEnd()
    #     self.messageWin.ScrollLines(self.messageWin.GetNumberOfLines()*5)
    #===========================================================================


    def OnNew(self, evt):
        self.canvas.DoNew()

    def OnLoadAssembly(self, evt):
        self.canvas.DoLoad(bAssembly=True, bSystem=False)
        
    def OnLoadSystem(self, evt):
        self.canvas.DoLoad(bAssembly=False, bSystem=True)

    def OnReload(self, evt):
        self.canvas.Reload()
        
    def OnReset(self, evt):
        self.canvas.Reset()
        
    def OnCompile(self, evt):
        self.canvas.Compile()

    def OnGenerateOritatamiSystem(self, evt):
        self.canvas.GenerateOritatamiSystem()

    def OnSaveShapeDef(self, event):
        self.canvas.DoSaveShapeDef()

    def OnSaveShapeDefAs(self, event):
        self.canvas.DoSaveShapeDef(bSaveAs=True)

    def OnSetSeed(self, event):
        self.canvas.DoSetSeed()

    def OnSaveSeed(self, event):
        self.canvas.DoSaveAssembly(bOnlySeed=True)

    def OnSaveSeedAs(self, event):
        self.canvas.DoSaveAssembly(bOnlySeed=True, bSaveAs=True)

    def OnSaveAssembly(self, event):
        self.canvas.DoSaveAssembly()

    def OnSaveAssemblyAs(self, event):
        self.canvas.DoSaveAssembly(bOnlySeed=False, bSaveAs=True)

    def OnSaveAll(self, event):
        self.canvas.DoSaveAll()
        
    def OnForward(self, event):
        self.canvas.DoForwardStep()
        
    def OnGo(self, event):
        self.canvas.DoGo()
        
    def OnCenter(self, event):
        self.canvas.ScrollToCoords((0,0))
        
    def OnZoomIn(self, event):
        self.canvas.ZoomThenRecenter(True)

    def OnZoomOut(self, event):
        self.canvas.ZoomThenRecenter(False)
    
    def OnMessageToggle(self, event):
        if self.itemTM.IsChecked():
            self.canvas.MessageToggle(True)
        else:
            self.canvas.MessageToggle(False)

    def OnDisplayToggle(self, event):
        if self.itemTD.IsChecked():
            self.canvas.DisplayToggle(True)
        else:
            self.canvas.DisplayToggle(False)

    def OnHelp(self, event):
        #info = wx.AboutDialogInfo()
        #info.SetName("Oritatami Simulator")
        #info.SetVersion("0.1.0")
        helpStr           = "                                         Oritatami Shape Maker\n"
        helpStr = helpStr + "                                           version 0.1.0\n\n"
        helpStr = helpStr + " This program allows a user to load or draw a shape, and then generate\n"
        helpStr = helpStr + " an Oritatami system which will self-assemble a scaled version of it in\n"
        helpStr = helpStr + " the OritatamiSim simulator.\n\n"
        helpStr = helpStr + " To generate a system, first use the mouse to select the connected set of\n"
        helpStr = helpStr + " points defining the shape.  Then, hit the 'Scale and rotate shape' button.\n"
        helpStr = helpStr + " If the points are connected, this will succeed in creating a scaled and\n"
        helpStr = helpStr + " slightly rotated version of the points.  Then, hit the 'Forward' or\n"
        helpStr = helpStr + " 'Fast Forward' button until all points in the scaled and rotated shape\n"
        helpStr = helpStr + " have been replaced by hexagons of seven points, and a Hamiltonian cycle\n"
        helpStr = helpStr + " has been drawn through all included points.  Finally, hit the 'Generate\n"
        helpStr = helpStr + " Oritatami system' button to select the names of the output files.\n\n"
        helpStr = helpStr + "    List of commands:\n\n"
        helpStr = helpStr + "      l : ('load') select shape definition file to load\n"
        helpStr = helpStr + "      s : save current shape definition\n"
        helpStr = helpStr + "      r : ('reset') reset current system back to shape definition\n"
        helpStr = helpStr + "      f : ('forward') simulate one step\n"
        helpStr = helpStr + "      g : ('go') begin fast forward simulation\n"
        helpStr = helpStr + "      q : ('quit') quit fast forward simulation\n"
        helpStr = helpStr + "      m : ('messges') toggle display of output messages\n"
        helpStr = helpStr + "      d : ('display') toggle updates to the display of the assembly\n"
        helpStr = helpStr + "      t : ('toggle') toggle display of grid lines\n"
        helpStr = helpStr + "      c : re-center grid\n"
        #helpStr = helpStr + "      b : ('breakpoints') set breakpoints\n"
        helpStr = helpStr + "      +/-: zoom in/out\n"
        helpStr = helpStr + "      <left mouse click>:  add point to shape definition\n"
        helpStr = helpStr + "      <right mouse click>:  remove point from shape definition\n"
        helpStr = helpStr + "      h : ('help') show this help dialog"
        helpStr = helpStr + "\n\n"
        #info.SetDescription(helpStr)
        #wx.AboutBox(info)
        helpPanel = wx.Panel(self.frame)
        helpText = wx.StaticText(helpPanel, label=helpStr)
        helpText.SetBackgroundColour((233,233,233))
        # add the panes to the manager
        self.frame._mgr.AddPane(helpText, wx.aui.AuiPaneInfo().Float().Caption("Oritatami Shape Maker Help"))
        self.frame._mgr.Update()
        
#----------------------------------------------------------------------------
#  Create the application and run it

app = OritatamiSimulatorApp()
app.MainLoop()

#---------------------------------------------------------------------------

