from ShapeMakerNodes import *
from SimulatorFramework import SimUtils
import math
import os
from collections import deque
import copy


        
class ONodeGrid:
    def __init__(self, width, height):
        self.MakeGrid(width, height)
        self.delayLength = 2    # Default delay of 2
        self.arity = 1          # Maximum number of bonds for a single bead
        self.seed = []
        self.beadCycle = []
        self.origBeadCycle = []
        self.currCyclePos = 0
        self.lastBeadLoc = None
        self.lastSeedLoc = None
        self.floppySegment = deque([])
        self.ONodeTypes = dict()
        self.shapePoints = []
        self.orderedList = []
        self.scaledAndRotatedList = []
        self.numTiles = 0
        
        # Hard code the bead types
        self.ONodeTypes["shapeNode"] = ShapeNodeType
        self.ONodeTypes["scaledNode"] = ScaledNodeType
        self.ONodeTypes["gadgetNode"] = GadgetNodeType

    def GetNumBeads(self):
        return self.numTiles
    
    def GetNumTiles(self):
        return self.numTiles

    def GetWidth(self):
        return self.width
    
    def GetHeight(self):
        return self.height

    def MakeGrid(self, width, height, bResetBeadNum=False):
        self.width = width
        self.height = height
        self._grid = [[None for x in range(width)] for y in range(height)]
        self.minX = 0 - int(math.floor(width/2))
        self.maxX = int(math.floor(width/2))
        self.minY = 0 - int(math.floor(height/2))
        self.maxY = int(math.floor(height/2))
        if (width % 2) == 0:
            self.minX = self.minX + 1
        if (height % 2) == 0:
            self.minY = self.minY + 1
        if bResetBeadNum == True:
            self.numTiles = 0

    def GetAllGadgets(self):
        retList = []
        for i in range(self.width):
            for j in range(self.height):
                loc = self._grid[i][j]
                if not loc is None:
                    coords = (i+self.minX, j+self.minY)
                    retList.append(coords)
        return retList

    def GetLoc(self, coords):
        if coords[0] < self.minX or coords[0] > self.maxX or coords[1] < self.minY or coords[1] > self.maxY:
            print("Growing grid because of access to " + str(coords))
            self.GrowGrid()
        return self._grid[coords[0]-self.minX][coords[1]-self.minY]
    
    def SetLoc(self, coords, onode):
        if coords[0] < self.minX or coords[0] > self.maxX or coords[1] < self.minY or coords[1] > self.maxY:
            self.GrowGrid()
        self._grid[coords[0]-self.minX][coords[1]-self.minY] = onode
        return True

    def GrowGrid(self):
        oldgrid = self._grid
        oldwidth = self.width
        oldheight = self.height
        oldminx = self.minX
        oldminy = self.minY
        self.MakeGrid(oldwidth + GRIDGROWTHINC, oldheight + GRIDGROWTHINC)
        for i in range(oldwidth):
            for j in range(oldheight):
                oldnode = oldgrid[i][j]
                self.SetLoc((i + oldminx, j + oldminy), oldnode)

    def AddNode(self, otype, coords, parentCoords, bWarnOnNoParent=True):
        childNode = ONode(otype, coords)
        if not self.GetLoc(coords) is None:
            print("ERROR: Trying to add another node at location " + str(coords))
            return
        self.SetLoc(coords, childNode)
        self.shapePoints.append(coords)
        self.numTiles = self.numTiles + 1
        
    def RemoveNode(self, coords):
        node = self.GetLoc(coords)
        if not node is None:
            
            self.shapePoints.remove(coords)
            self.SetLoc(coords, None)
            self.numTiles = self.numTiles - 1
        else:
            print("ERROR: Trying to remove nonexistent node at location" + str(coords))

    def AreNbrLocs(self, coords1, coords2):
        if (coords1 is None) or (coords2 is None):
            return False
        for n in ONBRS[1:]:
            nbrCoords = (coords1[0]+n[0], coords1[1]+n[1])
            if nbrCoords == coords2:
                return True
        return False

    def GetNbrLocs(self, coords, bEmpty=False):
        retlist = []
        for n in ONBRS[1:]:
            nbrCoords = (coords[0]+n[0], coords[1]+n[1])
            if bEmpty == False:
                retlist.append(nbrCoords)
            elif self.GetLoc(nbrCoords) is None:
                retlist.append(nbrCoords)
        return retlist

    def GetOccupiedNbrLocs(self, coords, bSkipParent=False, maxBonds=0):
        retlist = []
        for n in ONBRS[1:]:
            nbrCoords = (coords[0]+n[0], coords[1]+n[1])
            nbrNode = self.GetLoc(nbrCoords)
            if not nbrNode is None:
                #if (bSkipParent == False) or (nbrNode.child is None) or (not nbrNode.child.coords == coords):
                #    if nbrNode.NumBonds() <= maxBonds:
                retlist.append(nbrNode)
        return retlist

    def GetTopLeft(self):
        # Get leftmost point of the top row
        for i in reversed(range(self.height)):
            for j in range(self.width):
                n = self._grid[j][i]
                if not n is None:
                    y = i + self.minY
                    x = j + self.minX
                    return (x,y)
        return None

    def GetOrderedList(self):
        topLeft = self.GetTopLeft()     
        if topLeft is None:
            print("No points in the shape!")
            return None
        print("Topleft: " + str(topLeft))
        
        orderedPoints = [topLeft]
        n = 0
        while n < len(orderedPoints):
            nbrs = self.GetOccupiedNbrLocs(orderedPoints[n])
            for nbr in nbrs:
                if not nbr.coords in orderedPoints:
                    orderedPoints = orderedPoints + [nbr.coords]
            n = n + 1
        return orderedPoints

    def LoadOritatamiSystem(self, origFileName, origFileNameFullPath=None):
        fileName = origFileName
        if not origFileNameFullPath is None:
            fileName = origFileNameFullPath
        if len(fileName) < 1:
            return False
        elif os.path.isabs(fileName) == False:
            fileName = "./" + fileName
            fileName = os.path.abspath(fileName)

        try:
            os.stat(fileName)
            rFile = open(fileName, "r")
            print("Using oritatami system definition file '" + fileName + "'")
            lines = rFile.readlines()
            rFile.close()
    
            self.ONodeTypes = dict()
    
            for line in lines:
                if line.startswith("BeadTypes="):
                    try:
                        beadTypes = eval(line[len("BeadTypes="):])
                        for bead in beadTypes:
                            beadType = ONodeType(bead[0], bead[1], bead[2])
                            self.ONodeTypes[bead[0]] = beadType
                        self.FixBeadBondSymmetry(True)
                    except:
                        SimUtils.globalApp.ShowErrorDialog("Invalid bead type definitions.  See messages window.")
                        SimUtils.globalApp.OutputMessage("Invalid line '" + line + "'.  Bead types should be a comma separated list surrounded by '[' and ']'. Each bead type definition should similar to the following:  ['Name', 'Color', ['Name2', 'Name3']] where Name2 and Name3 are in a list of bead types that this type binds to.")
                        return False
                elif line.startswith("DelayLength="):
                    dlLine = line[len("DelayLength="):].strip()
                    self.delayLength = int(dlLine)
                    print("DelayLength: " + str(self.delayLength))
                    #if (self.delayLength > 2):
                        #MJP TODO: fix this issue!
                        #print("\n\nWARNING:  For delay longer than 2, newly extended beads could possibly bind to each other, but this is not yet implemented!!!\n")
                elif line.startswith("Arity="):
                    arityLine = line[len("Arity="):].strip()
                    self.arity = int(arityLine)
                    print("Arity: " + str(self.arity))
                elif line.startswith("BeadCycle="):
                    bcLine = line[len("BeadCycle="):].strip()
                    self.beadCycle = []
                    self.origBeadCycle = []
                    try:
                        self.beadCycle = eval(bcLine)
                        self.origBeadCycle = self.beadCycle
                    except:
                        SimUtils.globalApp.ShowErrorDialog("Invalid bead cycle definition.  See messages window.")
                        SimUtils.globalApp.OutputMessage("Invalid line '" + line + "'.  Bead cycle should be a comma separated list of bead type names surrounded by '[' and ']', similar to the following:  ['Name1', 'Name2', 'Name3'].")
                        return False
                    print("beadCycle: " + str(self.beadCycle))

            self.InitializeFloppySegment()
            SimUtils.globalApp.ResetBreakpoints()
            SimUtils.globalApp.systemDefinitionFile = origFileName
            SimUtils.globalApp.systemDefinitionFileFullPath = origFileNameFullPath
            SimUtils.globalApp.OutputMessage("Loaded bead type definitions from file '" + fileName + "'")
            print("Loaded bead set: " + str(self.ONodeTypes))
            return True;
        except:
            SimUtils.globalApp.ShowErrorDialog("\nERROR: Invalid oritatami system definition file: '" + fileName + "'\n")
            return False

    def LoadOritatamiAssembly(self, origFileName, bForceDefLoad=False):
        fileName = origFileName
        if len(fileName) < 1:
            return False
        elif os.path.isabs(fileName) == False:
            fileName = "./" + fileName
            fileName = os.path.abspath(fileName)

        lastLoc = None
        try:
            os.stat(fileName)
            rFile = open(fileName, "r")
            print("Using shape definition file '" + fileName + "'")
            lines = rFile.readlines()
            rFile.close()
            
            for line in lines:
                if line.startswith("Assembly="):
                    if self.LoadOritatamiAssemblyHelper(line) == False:
                        SimUtils.globalApp.OutputMessage("ERROR: Invalid shape definition file: '" + fileName + "'")
                        return False
        except:
            SimUtils.globalApp.ShowErrorDialog("\nERROR: Invalid shape definition file: '" + fileName + "'")
            return False
                
        SimUtils.globalApp.seedDefinitionFile = origFileName
        SimUtils.globalApp.OutputMessage("Loaded shape definition from file '" + SimUtils.globalApp.seedDefinitionFile + "'")
        return True

    def LoadOritatamiAssemblyHelper(self, seedString):
        seedString = seedString[len("Assembly="):].strip()
        try:
            self.shapePoints = []
            seedArr = eval(seedString)
            for b in seedArr:
                if b[0] in self.ONodeTypes:
                    self.shapePoints.append(b[1])
                else:
                    SimUtils.globalApp.OutputMessage("ERROR: Trying to add a bead of undefined type '" + str(b[0]) + "' at " + str(b[1]))
        except:
            SimUtils.globalApp.ShowErrorDialog("\nERROR: Invalid shape definition file.")
            return False
        return self.ResetToShapeDef()
        
    def ResetToShapeDef(self):
        print("Shape definition: " + str(self.shapePoints))
        self.MakeGrid(self.width, self.height, True)
        savedShapePoints = copy.deepcopy(self.shapePoints)
        self.shapePoints = []
        for p in savedShapePoints:
            self.AddNode(ShapeNodeType.name, p, None, False)

        return True
