from ShapeMakerUtils import *
from SimulatorFramework import TileType, Tile


class ONodeType(TileType):
    def __init__(self, name, color):
        TileType.__init__(self, name, 5, 5)
        self.color = color
        
    def __repr__(self):
        return self.name
    
    def __str__(self):
        return self.name

    def Output(self):
        outStr = "['" + self.name + "', '" + str(self.color) + "]"
        return outStr
    
ShapeNodeType = ONodeType("shapeNode", "BLUE")
ScaledNodeType = ONodeType("scaledNode", "BLACK")
GadgetNodeType = ONodeType("gadgetNode", "YELLOW")


class ONode(Tile):

    def __init__(self, otype, coords=None, parent=None):
        Tile.__init__(self, otype, coords)
        self.parent = parent
        self.child = None

    def __str__(self):
        outStr = "ONode[coords " + str(self.coords) + ", type " + str(self.type) + "]"
        return outStr

    def __repr__(self):
        outStr = "ONode[" + str(self.coords) + "]"
        return outStr
    
class NodeGadgetType(TileType):
    def __init__(self, name="defaultNodeGadgetType"):
        TileType.__init__(self, name, 5, 5)

DefaultNodeGadgetType = NodeGadgetType()

class NodeGadget(Tile):
    def __init__(self, coords):
        Tile.__init__(self, "defaultNodeGadgetType", coords)
        self.edges = []
        
    def __eq__(self, other):
        return isinstance(other, NodeGadget) and self.coords == other.coords

    def __hash__(self):
        return hash(self.coords)
    
    def AddEdges(self, edgeList):
        for e in edgeList:
            offset1 = ONBRS[e[0]]
            offset2 = ONBRS[e[1]]
            edge = [offset1, offset2]
            self.edges.append(edge)
            
    def RemoveEdges(self, edgeList):
        for e in edgeList:
            offset1 = ONBRS[e[0]]
            offset2 = ONBRS[e[1]]
            edge = [offset1, offset2]
            if edge in self.edges:
                self.edges.remove(edge)
                #print("Removed edge " + str(edge))
            else:
                edge = [offset2, offset1]
                if edge in self.edges:
                    self.edges.remove(edge)
                else:
                    globalApp.SetOutputMessage("ERROR: Trying to remove nonexistent edge!!")
            
    def GetNbrBorderPattern(self, nbrGadget):
        #print("GetNbrBorderPattern for gadget at " + str(self.coords) + " and nbr at " + str(nbrGadget.coords))
        nbr = nbrGadget.coords

        # Get the boundary edges from the neighbor's perspective
        nbrOffset = (self.coords[0]-nbr[0], self.coords[1]-nbr[1])
        boundaryEdge = SHAREDEDGES[nbrOffset]
        sharedVerts = (ONBRINDEX[boundaryEdge[0]], ONBRINDEX[boundaryEdge[1]])
        fullEdge = [ONBRS[sharedVerts[0]],ONBRS[sharedVerts[1]]]
        fullEdgeReversed = [fullEdge[1], fullEdge[0]]
        V = [[ONBRS[sharedVerts[0]],(0,0)],[(0,0),ONBRS[sharedVerts[1]]]]
        VReversed = [[(0,0),ONBRS[sharedVerts[0]]],[ONBRS[sharedVerts[1]],(0,0)]]
        #print("Nbr's shared verts: " + str(sharedVerts) + ", full edge: " + str(fullEdge) + ", and nbr's edges " + str(nbrGadget.edges))
        #print("V: " + str(V))
        flippedVerts = NBRFLIP[(sharedVerts[0], sharedVerts[1])]
        if (fullEdge in nbrGadget.edges) or (fullEdgeReversed in nbrGadget.edges):
            # Return just this edge, from the caller's perspective
            #print("HAS EDGE")
            return [flippedVerts]
        elif (V[0] in nbrGadget.edges and V[1] in nbrGadget.edges) or (VReversed[0] in nbrGadget.edges and VReversed[1] in nbrGadget.edges):
            #print("HAS V")
            return [(flippedVerts[0], 8), (8, flippedVerts[1])]
        else:
            return [(9,9)]
            #globalApp.SetOutputMessage("Encountered unrecognized edge pattern on gadget border.  Aborting.")
            #print("HAS NEITHER: Gotta figure out what to return...")
            #return None

    def DrawPolygon(self, pdc, canvas):
        pdc.SetBrush(canvas.ColoredBrush("YELLOW"))
        pdc.SetPen(canvas.gridPen)
        polygon = []
        for n in ONBRS[1:]:
            no = (self.coords[0]+n[0], self.coords[1]+n[1])
            px = canvas.GetPixelsForCoords(no[0], no[1])
            polygon.append(px)
        polygon.append(polygon[0])
        pdc.DrawPolygon(polygon)

    def Draw(self, pdc, canvas):
        #print("time to draw gadget with edges " + str(self.edges))
        pdc.SetBrush(canvas.ColoredBrush("YELLOW"))
        pdc.SetPen(canvas.cyclePen)
        for e in self.edges:
            loc1 = (self.coords[0]+e[0][0], self.coords[1]+e[0][1])
            loc2 = (self.coords[0]+e[1][0], self.coords[1]+e[1][1])
            px1 = canvas.GetPixelsForCoords(loc1[0], loc1[1])
            px2 = canvas.GetPixelsForCoords(loc2[0], loc2[1])
            pdc.DrawLine(px1[0], px1[1], px2[0], px2[1])
            pdc.DrawCircle((px1[0], px1[1]), 10)
            pdc.DrawCircle((px2[0], px2[1]), 10)
        pdc.SetBrush(canvas.ColoredBrush("BLACK"))
        px1 = canvas.GetPixelsForCoords(self.coords[0], self.coords[1])
        pdc.DrawCircle((px1[0], px1[1]), 10)
        return True

class GadgetSet:
    def __init__(self):
        self.gadgets = dict()
    
    def Add(self, nodeGadget):
        self.gadgets[nodeGadget.coords] = nodeGadget

    def HasLoc(self, coords):
        if coords in self.gadgets.keys():
            return True
        return False

    def Get(self, coords):
        if self.HasLoc(coords):
            return self.gadgets[coords]
        return None

    def GetNbrs(self, coords):
        if self.HasLoc(coords):
            snbrList = []
            for sn in SCALEDNBRS[1:]:
                snbr = (coords[0]+sn[0], coords[1]+sn[1])
                if self.HasLoc(snbr):
                    snbrList.append(self.Get(snbr))
            return snbrList
        return None
    
    def GetAbstractNbrPattern(self, coords):
        if self.HasLoc(coords):
            nbrPattern = []
            for sn in SCALEDNBRS[1:]:
                abstractPattern = "_"
                snbr = (coords[0]+sn[0], coords[1]+sn[1])
                if self.HasLoc(snbr):
                    pattern = self.Get(coords).GetNbrBorderPattern(self.Get(snbr))
                    #print("got pattern: " + str(pattern))
                    if (9,9) in pattern:
                        abstractPattern = "D"
                    else:
                        if len(pattern) == 1:
                            abstractPattern = "S"
                        else:
                            abstractPattern = "V"
                nbrPattern.append(abstractPattern)
            return nbrPattern
        return None
    
    def ExtendCycle(self, coords, rot, reflect, changeList):
        #print("In ExtendCycle about to implement changes: " + str(changeList))
        for gadgetChanges in changeList:
            nbrNum = RotateNbrGadgetNum(gadgetChanges[0], rot)
            addList = gadgetChanges[1]
            removeList = gadgetChanges[2]
            
            nbrOffset = SCALEDNBRS[nbrNum]
            gadgetCoords = (coords[0]+nbrOffset[0], coords[1]+nbrOffset[1])

            if not self.HasLoc(gadgetCoords):
                globalApp.SetOutputMessage("ERROR: In ExtendCycle, no gadget at " + str(gadgetCoords))
                return False

            #print("  in loc " + str(gadgetCoords) + ":")
                 
            rotAdd = RotateEdges(addList, rot)
             
            #print("        adding: " + str(addList) + " rotated: " +  str(rotAdd))
            self.Get(gadgetCoords).AddEdges(rotAdd)
            
            rotRem = RotateEdges(removeList, rot)

            #print("       removing: " + str(removeList) + " rotated: " + str(rotRem))
            self.Get(gadgetCoords).RemoveEdges(rotRem)
            
        return True
